@@ -22,7 +22,7 @@ from subprocess import Popen, PIPE, STDOUT
 import inspect
 from collections import deque
 
-from microarch import get_all_data, is_superset_of, set_diff, extensions_to_names, get_compiler_flags_arch_id
+from microarch import get_all_data, get_all_data_from_marchid, is_superset_of, set_diff, extensions_to_names, get_compiler_flags_arch_id
 
 DEFAULT_ORGANIZATION_NAME = 'k-nuth'
 DEFAULT_LOGIN_USERNAME = 'fpelliccioni'
@@ -540,78 +540,75 @@ def get_builder(recipe_dir, args=None):
 
 
 def march_conan_manip(conanobj):
-    conanobj.output.info("KnuthConanFile.march_conan_manip() - 1")
     if conanobj.settings.arch != "x86_64":
-        return (None, None, None)
-
-    conanobj.output.info("KnuthConanFile.march_conan_manip() - 2")
+        return (None, None, None, None)
 
     if conanobj.options.get_safe("march_id") is None:
-        return (None, None, None)
-
-    conanobj.output.info("KnuthConanFile.march_conan_manip() - 3")
+        return (None, None, None, None)
 
     if conanobj.options.get_safe("march_strategy") is None:
-        return (None, None, None)
+        return (None, None, None, None)
 
-    conanobj.output.info("KnuthConanFile.march_conan_manip() - 4")
-
-    conanobj.output.info(f"os:               {str(conanobj.settings.os)}")
-    conanobj.output.info(f"compiler:         {str(conanobj.settings.compiler)}")
-    conanobj.output.info(f"compiler.version: {float(str(conanobj.settings.compiler.version))}")
-
-    conanobj.march_data = get_all_data(str(conanobj.settings.os),
-                                       str(conanobj.settings.compiler),
-                                       float(str(conanobj.settings.compiler.version)))
-
-    conanobj.output.info("KnuthConanFile.march_conan_manip() - 5")
-
-
-    march_from = 'taken from cpuid'
+    conanobj.march_from_cpuid = True
+    march_from_str = 'taken from cpuid'
     march_id = None
     march_names = None
     march_flags = None
 
     if conanobj.options.march_id == "_DUMMY_":
-        conanobj.output.info("KnuthConanFile.march_conan_manip() - 6")
+        conanobj.march_data = get_all_data(str(conanobj.settings.os),
+                                        str(conanobj.settings.compiler),
+                                        float(str(conanobj.settings.compiler.version)))
 
         if conanobj.options.march_strategy == "optimized":
-            conanobj.output.info("KnuthConanFile.march_conan_manip() - 7")
             march_id = conanobj.march_data['comp_marchid']
             march_names = conanobj.march_data['comp_names']
             march_flags = conanobj.march_data['comp_flags']
         elif conanobj.options.march_strategy == "download_if_possible":
-            conanobj.output.info("KnuthConanFile.march_conan_manip() - 8")
             march_id = conanobj.march_data['comp_marchid']
             march_names = conanobj.march_data['comp_names']
             march_flags = conanobj.march_data['comp_flags']
             exts = conanobj.march_data['comp_exts']
             level3_exts = conanobj.march_data['level3_exts']
             if is_superset_of(exts, level3_exts):
-                conanobj.output.info("KnuthConanFile.march_conan_manip() - 9")
                 march_id = conanobj.march_data['level3_marchid']
                 march_names = conanobj.march_data['level3_names']
                 march_flags = conanobj.march_data['level3_flags']
         elif conanobj.options.march_strategy == "download_or_fail":
-            conanobj.output.info("KnuthConanFile.march_conan_manip() - 10")
             march_id = conanobj.march_data['comp_marchid']
             march_names = conanobj.march_data['comp_names']
             march_flags = conanobj.march_data['comp_flags']
             exts = conanobj.march_data['comp_exts']
             level3_exts = conanobj.march_data['level3_exts']
             if not is_superset_of(exts, level3_exts):
-                conanobj.output.info("KnuthConanFile.march_conan_manip() - 11")
-                return (None, None, None)
+                return (None, None, None, None)
 
         conanobj.options.march_id = march_id
     else:
-        conanobj.output.info("KnuthConanFile.march_conan_manip() - 12")
-        march_id = conanobj.options.march_id
-        march_from = 'user defined'
-        #TODO(fernando): check for march_id errors
+        march_id = str(conanobj.options.march_id)
+        conanobj.march_from_cpuid = False
+        march_from_str = 'user defined'
 
-    conanobj.output.info("KnuthConanFile.march_conan_manip() - 13")
-    conanobj.output.info("Detected microarchitecture ID (%s): %s" % (march_from, march_id))
+        conanobj.march_data = get_all_data_from_marchid(
+                                        march_id,
+                                        str(conanobj.settings.os),
+                                        str(conanobj.settings.compiler),
+                                        float(str(conanobj.settings.compiler.version)))
+
+        march_names = conanobj.march_data['user_names']
+        march_flags = conanobj.march_data['user_flags']
+
+        #TODO(fernando): marchid errors?
+        #TODO(fernando): march_strategy ??
+
+
+
+        # if conanobj.options.march_strategy == "optimized":
+        #     march_names = conanobj.march_data['comp_names']
+        #     march_flags = conanobj.march_data['comp_flags']
+        # elif conanobj.options.march_strategy == "download_if_possible":
+
+    conanobj.output.info("Detected microarchitecture ID (%s): %s" % (march_from_str, march_id))
 
     return (march_id, march_names, march_flags)
 
@@ -666,9 +663,6 @@ def get_requirements_from_file():
     return []
 
 class KnuthConanFile(ConanFile):
-    # def __init__(self):
-    #     self.march_data = None
-
     @property
     def conan_req_version(self):
         return get_conan_req_version(self.recipe_dir())
@@ -691,6 +685,10 @@ class KnuthConanFile(ConanFile):
     #         self.output.warn("**** The recipe does not implement the march_id option. ****")
 
     def validate(self):
+
+        self.output.info(f"validate() self.march_data: {self.march_data}")
+        self.output.info(f"validate() self.march_from_cpuid: {self.march_from_cpuid}")
+
         # ConanFile.validate(self)
         if self.conan_req_version != None and Version(conan_version) < Version(self.conan_req_version):
             raise ConanInvalidConfiguration("Conan version should be greater or equal than %s. Detected: %s." % (self.conan_req_version, conan_version))
@@ -702,30 +700,37 @@ class KnuthConanFile(ConanFile):
             if self.options.get_safe("march_strategy") is None:
                 raise ConanInvalidConfiguration("The recipe does not implement the march_strategy option.")
 
-            if self.options.march_strategy == "download_or_fail":
-                exts = conanobj.march_data['comp_exts']
-                level3_exts = conanobj.march_data['level3_exts']
-                if not is_superset_of(exts, level3_exts):
-                    exts_diff = set_diff(cpu_exts, comp_exts)
-                    exts_names = extensions_to_names(diff)
+            if self.march_from_cpuid:
+                if self.options.march_strategy == "download_or_fail":
+                    exts = self.march_data['comp_exts']
+                    level3_exts = self.march_data['level3_exts']
+                    if not is_superset_of(exts, level3_exts):
+                        exts_diff = set_diff(level3_exts, exts)
+                        exts_names = extensions_to_names(exts_diff)
+                        exts_str = " ".join(exts_names)
+                        raise ConanInvalidConfiguration(f"The detected micro-architecture of your system is not compatible with x86-64-v3 (Check https://en.wikipedia.org/wiki/X86-64#Microarchitecture_levels).\nThe following extensions are not supported by your system: {exts_str}.\nThis error is generated because you chose march_strategy = download_or_fail.")
+            else:
+                if self.march_data['user_marchid_valid']:
+                    raise ConanInvalidConfiguration(f"{self.options.get_safe("march_id")} is not a valid micro-architecture id (march_id option).")
+
+                exts = self.march_data['user_exts']
+                exts_filtered = self.march_data['user_exts_filtered']
+                if not is_superset_of(exts_filtered, exts):
+                    exts_diff = set_diff(exts, exts_filtered)
+                    exts_names = extensions_to_names(exts_diff)
                     exts_str = " ".join(exts_names)
-                    raise ConanInvalidConfiguration(f"The detedted micro-architecture of your system is not compatible with x86-64-v3 (Check https://en.wikipedia.org/wiki/X86-64#Microarchitecture_levels).\nThe following extensions are not supported by your system: {exts_str}.\nThis error is generated because you chose march_strategy = download_or_fail.")
+                    raise ConanInvalidConfiguration(f"{self.options.get_safe("march_id")} is not compatible with your compiler.\nThe following extensions are not supported by your compiler: {exts_str}.")
 
 
     def configure(self, pure_c=False):
         # if self.conan_req_version != None and Version(conan_version) < Version(self.conan_req_version):
         #     raise Exception ("Conan version should be greater or equal than %s. Detected: %s." % (self.conan_req_version, conan_version))
 
-        self.output.info("KnuthConanFile.configure() - 1")
         ConanFile.configure(self)
-        self.output.info("KnuthConanFile.configure() - 2")
 
         if pure_c:
             del self.settings.compiler.libcxx               #Pure-C Library
 
-        self.output.info("KnuthConanFile.configure() - 3")
-
-
         if self.options.get_safe("currency") is not None:
             self.options["*"].currency = self.options.currency
             self.output.info("Compiling for currency: %s" % (self.options.currency,))
@@ -734,34 +739,19 @@ class KnuthConanFile(ConanFile):
             self.options["*"].db = self.options.db
             self.output.info("Compiling for DB: %s" % (self.options.db,))
 
-        self.output.info("KnuthConanFile.configure() - 4")
-
         # self._warn_missing_options()
 
         if self.settings.arch == "x86_64":
-            self.output.info("KnuthConanFile.configure() - 5")
-
             (march_id, march_names, march_flags) = march_conan_manip(self)
-
-            self.output.info(f"KnuthConanFile.configure() - march_id: {march_id} - march_names: {march_names} - march_flags: {march_flags}")
-
-            self.output.info("KnuthConanFile.configure() - 6")
+            if march_names is not None:
+                self.output.info(f"The package is being compiled for a system that supports: {', '.join(march_names)}")
 
             self.options["*"].march_id = march_id
-
-            self.output.info("KnuthConanFile.configure() - 7")
-
             self.options["*"].march_strategy = self.options.get_safe("march_strategy")
 
-
-            self.output.info("KnuthConanFile.configure() - 8")
-
-
             if self.options.get_safe("march_id") is not None:
                 self.options.march_id = march_id
 
-            self.output.info("KnuthConanFile.configure() - 9")
-
             #TODO(fernando)
             # if self.options.get_safe("march_id") is not None:
             #     self.output.info("Building microarchitecture ID: %s" % march_id)
